<?php
/**
 * Plugin Messages (addon for plugin Adm_redacteurs)
 * Version  : 1.4 (2021/01/03)
 * Compatibility : Guppy v5.x et v6.00.00 et Adm_redacteurs v6.14+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

require_once ADMR_HELPERS_PATH.'/display_error_helper.php';

class Display_error_messages_helper extends Display_error_helper
{
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    protected $CI; // object global
    protected $_lang = array();         // Données de langue
    protected $_config = array();       // Données de configuration
    protected $_writer_rights = array(); // Droits du rédacteur
    //
    private $_error_type = '';           // Type d'erreur
    private $_addon = 'messages';        // Nom de l'addon   
    
    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */    
    public function __construct($writer_rights)
    {        
        parent :: __construct($writer_rights);

        $this -> CI -> load_addon_helper('messages', 'messages_infos_helper');
        
        $this -> CI -> load_addon_lang('messages');
        $this -> CI -> load_addon_config('messages');
        
        $this -> _lang = $this -> CI -> get_lang();
        $this -> _config = $this -> CI -> get_config();       
    }

	// --------------------------------------------------------------------

    public function index()
    {        
        parent :: index();
    }

     
	// --------------------------------------------------------------------

	/**
	 * Renvoie le message d'erreur correspondant au type d'erreur
     * 
	 * @access protected
     * @param string Type d'erreur
	 * @return string Message d'erreur
	 */    
    protected function _get_error_message($error_type)
    {
        return parent :: _get_error_message($error_type);
    }
        
	// --------------------------------------------------------------------

	/**
	 * Définit les messages d'erreur associé à leur type
     * 
	 * @access protected
	 * @return array Tableau contenant les messages d'erreur sous la forme array('type' => 'message')
	 */
    protected function _set_errors_messages()
    {
        $errors_messages = array_merge(
            parent :: _set_errors_messages(),
            array(
                'compatibility' => sprintf(ucfirst($this -> _lang['messages_is_compatible_only_with_adm_redacteurs_version_greater_than_%s']), '6.7.0')
            )
        );
        
        return $errors_messages;
    } 
       
	// --------------------------------------------------------------------

    /*
     * Définit et renvoie le titre de la page 
     *
     * @param array Données à faire passer à la vue
     * @return void
     */        
    public function set_title_page()
    {
        return parent :: set_title_page();
    }
    
	// --------------------------------------------------------------------

	/**
	 * définit et renvoie la description de la page ainsi que les recommendations
     * 
	 * @return string titre de la page
	 * @access public
	 */	
    public function set_infos_page_content()
    {		
        return parent :: set_infos_page_content();
    }

	// --------------------------------------------------------------------

	/**
	 * définit et renvoie des ajouts (js, css, code php) dans la partie <head> de la page 
     *
	 * @access public
	 */	
    public function set_header_addons()
    {
        $parent = parent :: set_header_addons();
        
        $js = '';
//        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/display_error.js"></script>'."\n";
             
        $css = '';
//        $css .= '<link rel="stylesheet" type="text/css" href="'.ADMR_CSS_PATH.'/display_error.css" />';

        return $parent.$css.$js;

    }
        
	// --------------------------------------------------------------------

    /**
     * Définit le contenu du menu qui est affiché dans le bandeau en haut de la page
     *
     * @return array Tableau contenant les items du menu (vide par défaut)
     */
    public function set_menu_plugin_items()
    {        
        return array();
    }

	// --------------------------------------------------------------------

    /**
     * Définit le contenu des informations du plugin qui sont affichées en bas de la page
     *
     * @return array Tableau contenant les informations du plugin sous la forme array('version' => , 'licence' => )
     */
    public function set_infos_plugin()
    {
        return Messages_infos_helper :: set_plugin_infos($this -> _lang);
    }
   
	// --------------------------------------------------------------------
    
} // End Class